/* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
 * ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * Copyright (C) 1994  Microsoft Corporation.  All Rights Reserved.
 *
 *  MODULE:   dcomp_s.h
 *
 *  PURPOSE:
 *	  main server-side header file for the Distributed Compression sample
 *
 *  COMMENTS:
 */

#include	"funcs_s.h"
#include	"service.h"
#include	"regdb.h"


// Type definitions

typedef struct				// Key value data required by RegSetValueEx
{
	LPCTSTR	lpszValueName;	// address of value to set 
	DWORD	fdwType;		// flag for value type 
	LPBYTE	*lplpbData;		// address of value data 
	DWORD	cbData;			// size of value data 
} KEYVALUE;


// Function prototypes

VOID ExamineAccessToken(HANDLE hAccessToken);
BOOL SetPrivilegeInAccessToken(VOID);
VOID InitializeWellKnownSIDs(VOID);

void		InitDecompQueues(void);
void		InitCompQueues(void);
void		DecompBunchPass(void);
void		CompBunchPass(void);
DWORD WINAPI PerfMonStart(LPVOID dummy);
LONG			FindBunch(LPBUNCHQUEUE lpbq, LONG lBunchId);
LONG			GrabBunch(LPBUNCHQUEUE lpbq, LONG lBunchId);
LONG			GrabFirstBunch(LPBUNCHQUEUE lpbq);
LONG			GrabEmptyBunch(LPBUNCHQUEUE lpbq);
LONG			QueueBunch(LPBUNCHQUEUE lpbq, LONG lSlot);
LONG			QueueBunchAtHead(LPBUNCHQUEUE lpbq, LONG lSlot);
void			ClearDownBunch(LPBUNCHQUEUE lpbq, LONG lSlot);
LONG			ReserveSlot(LPBUNCHQUEUE lpbq);
LONG			UnreserveSlot(LPBUNCHQUEUE lpbq);
LONG		DecompBunch
(
	LONG			lBunchId,
	LONG			lBunchLen,
	LPBITMAPINFO	lpbiIn,
	LPBITMAPINFO	lpbiOut,
	LONG			alIn[],
	LONG			lInDataLen,
	BYTE			lpbInData[],
	BYTE			lpbOutData[]
);

LONG		CompBunch
(
	LONG			lBunchId,
	LONG			lBunchLen,
	LPBITMAPINFO	lpbiIn,
	LPBITMAPINFO	lpbiOut,
	BYTE			lpbInData[],
	LONG			alOut[],
	DWORD			dwRetFlags[],
	LONG			*lplOutDataLen,
	BYTE			lpbOutData[]
);

// Convenience macros

#define	QFull(bq)		((bq).nQCnt >= (bq).nQLen)
#define	QEmpty(bq)		((bq).nQHead == DCOMP_QEND)


// Bunch queue values - these queue lengths only determine space allocated,
// i.e. maximum queue length.  The working queue lengths are determined at
// run time (in StartCodecThreads).

#define		DECOMPBUNCHINQLEN		5
#define		DECOMPBUNCHOUTQLEN		5
#define		COMPBUNCHINQLEN			5
#define		COMPBUNCHOUTQLEN		5
#define		BUNCHPOOLLEN			(DECOMPBUNCHINQLEN + DECOMPBUNCHOUTQLEN\
									 + COMPBUNCHINQLEN + COMPBUNCHOUTQLEN)

// Queue status values

#define	DCOMP_QEMPTY			0L
#define	DCOMP_QQUEUED			1L
#define	DCOMP_QBUSY			2L

#define	DCOMP_QEND			-1L
#define	DCOMP_QERR			-1L


// Default configuration key values

#define	DEFAULT_BUSY_THRESHOLD		25
#define	DEFAULT_BUSY_PERIOD			60
#define	DEFAULT_PERFMON_ENA_FLAG	TRUE
#define	DEFAULT_VIDEO_DISP_FLAG		TRUE
#define	DEFAULT_USER_PERMS_CMPTR	TEXT("")
#define DEFAULT_DEBUG_OUTPUT_FILE	TEXT("")


// Other constants

#define ACL_BUFFER_SIZE     		1024

#define	MAX_NUM_PROCESSORS			32
									// based on size of dwActiveProcessorMask
									// element of the SYSTEM_INFO structure


// Global variables

#ifdef DEFINE_GLOBALS

#define GLOBAL   

CRITICAL_SECTION	csQPool;

BUNCH			abnchQPool[BUNCHPOOLLEN];
BUNCHQUEUE		bqDecompBunchInQ = {abnchQPool, &csQPool, DCOMP_QEND, {0}, 0L,
											DECOMPBUNCHINQLEN, BUNCHPOOLLEN};

BUNCHQUEUE		bqDecompBunchOutQ = {abnchQPool, &csQPool, DCOMP_QEND, {0}, 0L,
											DECOMPBUNCHOUTQLEN, BUNCHPOOLLEN};

BUNCHQUEUE		bqCompBunchInQ = {abnchQPool, &csQPool, DCOMP_QEND, {0}, 0L,
											COMPBUNCHINQLEN, BUNCHPOOLLEN};

BUNCHQUEUE		bqCompBunchOutQ = {abnchQPool, &csQPool, DCOMP_QEND, {0}, 0L,
											COMPBUNCHOUTQLEN, BUNCHPOOLLEN};


// Application configuration values

// Double indirection used for DWORD values to allow uniform handling with
// SZ values

DWORD			gdwBusyThreshold    = DEFAULT_BUSY_THRESHOLD;
DWORD			gdwBusyPeriod       = DEFAULT_BUSY_PERIOD;
TCHAR			gabUserPermsCmptr[] = DEFAULT_USER_PERMS_CMPTR;
DWORD			gdwPerfMonEna       = DEFAULT_PERFMON_ENA_FLAG;
TCHAR			gabDbgOutFile[]     = DEFAULT_DEBUG_OUTPUT_FILE;

LPDWORD			glpdwBusyThreshold  = &gdwBusyThreshold;
LPDWORD			glpdwBusyPeriod     = &gdwBusyPeriod;
LPTSTR			gpszUserPermsCmptr  = gabUserPermsCmptr;
LPDWORD			glpdwPerfMonEna     = &gdwPerfMonEna;
LPTSTR			gpszDbgOutFile      = gabDbgOutFile;


// Table of configuration key values, initialised to the defaults

KEYVALUE gakvConfig[] =
{																 
	{TEXT("BusyThreshold%"),			REG_DWORD,
		(LPBYTE*) &glpdwBusyThreshold,		sizeof gdwBusyThreshold		},

	{TEXT("BusyPeriodSecs"),			REG_DWORD,
		(LPBYTE*) &glpdwBusyPeriod,			sizeof gdwBusyPeriod		},

	{TEXT("UserPermissionsComputer"),	REG_SZ,
		(LPBYTE*) &gpszUserPermsCmptr,		sizeof gabUserPermsCmptr	},

	{TEXT("PerfMonEnable"),				REG_DWORD,
		(LPBYTE*) &glpdwPerfMonEna,			sizeof gdwPerfMonEna		},

	{TEXT("DebugOutputFile"),			REG_SZ,
		(LPBYTE*) &gpszDbgOutFile,			sizeof gabDbgOutFile		},

	{NULL}
};

int	 		gfDebug = TRUE;

#else

// only dcomp_s.c defines DEFINE_GLOBALS,
// all other references to these variables are extern

#define GLOBAL extern

#endif

	
GLOBAL	int					gfDebug;
GLOBAL	CRITICAL_SECTION	csStderr;

GLOBAL	DWORD				gdwOSPlatformId;

GLOBAL	BUNCH				abnchQPool[];
GLOBAL	CRITICAL_SECTION	csQPool;

GLOBAL	BUNCHQUEUE			bqDecompBunchInQ;
GLOBAL	BUNCHQUEUE			bqDecompBunchOutQ;

GLOBAL	BUNCHQUEUE			bqCompBunchInQ;
GLOBAL	BUNCHQUEUE			bqCompBunchOutQ;

GLOBAL	LONG				nCompBunchInQHead;
GLOBAL	LONG				nCompBunchOutQCnt;

GLOBAL	HANDLE				hCodec[MAX_NUM_PROCESSORS];
GLOBAL	DWORD				dwIDCodec[MAX_NUM_PROCESSORS];

GLOBAL	HANDLE				hPerfMon;
GLOBAL	DWORD				dwIDPerfMon;

GLOBAL	int					gfInitPending;

GLOBAL	BOOL				bDebug;
GLOBAL	BOOL				gfServerComputerBusy;
GLOBAL	SYSTEM_INFO			gsiSysInfo;
GLOBAL	DWORD				gdwNumActiveProcessors;

GLOBAL	DWORD				gdwBusyThreshold;
GLOBAL	DWORD				gdwBusyPeriod;
GLOBAL	TCHAR				gabUserPermsCmptr[];
GLOBAL	DWORD				gdwPerfMonEna;
GLOBAL	DWORD				gdwVideoDispEna;
GLOBAL	TCHAR				gabDbgOutFile[];

GLOBAL	LPDWORD				glpdwBusyThreshold;
GLOBAL	LPDWORD				glpdwBusyPeriod;
GLOBAL	LPTSTR				gpszUserPermsCmptr;
GLOBAL	LPDWORD				glpdwPerfMonEna;
GLOBAL	LPDWORD				glpdwVideoDispEna;
GLOBAL	LPTSTR				gpszDbgOutFile;

GLOBAL	KEYVALUE			gakvConfig[];

GLOBAL	TCHAR				gszComputerName[MAX_COMPUTERNAME_LENGTH + 1];
											// name of the local computer

HANDLE						gahClearQueuesEvent[MAX_NUM_PROCESSORS];
DWORD						gdwNumCodecThreads;

